[![FactSet](https://raw.githubusercontent.com/factset/enterprise-sdk/main/docs/images/factset-logo.svg)](https://www.factset.com)

# FactSet Intraday Tick History client library for Python

[![API Version](https://img.shields.io/badge/api-v1.0.0-blue)](https://www.factset.com/hubfs/Website_Downloads/Exchange%20DataFeed/data%20service%20manual%202.0b.pdf)
[![PyPi](https://img.shields.io/pypi/v/fds.sdk.FactSetIntradayTickHistory)](https://pypi.org/project/fds.sdk.FactSetIntradayTickHistory/)
[![Apache-2 license](https://img.shields.io/badge/license-Apache2-brightgreen.svg)](https://www.apache.org/licenses/LICENSE-2.0)


<p>FactSet’s Tick History provides cost-effective access to real-time and delayed global exchange data. Proprietary technology normalizes over 200 global exchanges and 19 tick history fields. Asset types integrated include equities, futures, options, fixed income, mutual funds, ETFs, indices, commodities, and FX rates.</p><p>Tick History provides dynamic access to tick data for a specific security intra-day or historically via standard HTTPS protocol. History is available for all quotes and trades for a trailing 6 months or 1-year, respectively. Quotes and trade data for Options is available for 30 days. Tick History data is also available in a separate end-of-day feed.</p>
 


This Python package is automatically generated by the [OpenAPI Generator](https://openapi-generator.tech) project:

- API version: 1.0.0
- SDK version: 0.21.13
- Build package: org.openapitools.codegen.languages.PythonClientCodegen

## Requirements

* Python >= 3.7

## Installation

### Poetry

```shell
poetry add fds.sdk.utils fds.sdk.FactSetIntradayTickHistory==0.21.13
```

### pip

```shell
pip install fds.sdk.utils fds.sdk.FactSetIntradayTickHistory==0.21.13
```

## Usage

1. [Generate authentication credentials](../../../../README.md#authentication).
2. Setup Python environment.
   1. Install and activate python 3.7+. If you're using [pyenv](https://github.com/pyenv/pyenv):

      ```sh
      pyenv install 3.9.7
      pyenv shell 3.9.7
      ```

   2. (optional) [Install poetry](https://python-poetry.org/docs/#installation).
3. [Install dependencies](#installation).
4. Run the following:

> [!IMPORTANT]
> The parameter variables defined below are just examples and may potentially contain non valid values. Please replace them with valid values.

### Example Code

```python
from fds.sdk.utils.authentication import ConfidentialClient

import fds.sdk.FactSetIntradayTickHistory
from fds.sdk.FactSetIntradayTickHistory.api import first_trade_api
from fds.sdk.FactSetIntradayTickHistory.models import *
from dateutil.parser import parse as dateutil_parser
from pprint import pprint

# See configuration.py for a list of all supported configuration parameters.

# Examples for each supported authentication method are below,
# choose one that satisfies your use case.

# (Preferred) OAuth 2.0: FactSetOAuth2
# See https://github.com/FactSet/enterprise-sdk#oauth-20
# for information on how to create the app-config.json file
#
# The confidential client instance should be reused in production environments.
# See https://github.com/FactSet/enterprise-sdk-utils-python#authentication
# for more information on using the ConfidentialClient class
configuration = fds.sdk.FactSetIntradayTickHistory.Configuration(
    fds_oauth_client=ConfidentialClient('/path/to/app-config.json')
)

# Basic authentication: FactSetApiKey
# See https://github.com/FactSet/enterprise-sdk#api-key
# for information how to create an API key
# configuration = fds.sdk.FactSetIntradayTickHistory.Configuration(
#     username='USERNAME-SERIAL',
#     password='API-KEY'
# )

# Enter a context with an instance of the API client
with fds.sdk.FactSetIntradayTickHistory.ApiClient(configuration) as api_client:
    # Create an instance of the API class
    api_instance = first_trade_api.FirstTradeApi(api_client)
    id = "id_example" # str | Requested symbol or security. The symbol can be a FactSet exchange symbol, CUSIP, or SEDOL. **NOTE**: Only one identifier can be requested per request.<P>**TRY IT OUT** - Choose One from - BABA-USA,WALMEX-MX,7203-TKS,VOD-LON,NPN-JSE,MABAX,NZF,SPY-USA,AGG-USA,GLD-USA,AOR-USA,MNA-USA,UUP-USA,SP50-SPX,ESX-STX,XAO-ASX,WD-MSX,NG00-USA,GC00-USA,CC00-USA,C00-USA,FC00-USA,ER00-USA,EURUSD-FX1,USDMXN-FX1,AUDJPY-FX1,EURCZK-FX1,USDILS-FX1,USDZAR-FX1,US10YY-TU1,FDS#190621C00145000-USA,FDS#190621P00145000-USA
    req_id = "req_id_example" # str | Request Identification String. Can be used by the application to keep track of requests. The id is not used by this service, however, it is included in the XML response. (optional)
    start_date = "start_date_example" # str | This is the start date for the Snapshot History. Requests should be made in the format YYYYMMDD Default is current business day. Field can be sd or date. TRY IT OUT Cannot use current day. (optional)
    start_time = "040000" # str | This is the start time for the snapshot history requested. Requests should be made in a **HHMMSSS** format.Range from 000000 - 235959. **TRY IT OUT** Cannot use current day (optional) if omitted the server will use the default value of "040000"
    format = "XML" # str | The format of the output file. TRY IT OUT Choose from JSON, CSV, CSV_NO_HEADER (optional) if omitted the server will use the default value of "XML"
    include_request = "false" # str | Flag to include request id and key in the CSV output. (optional) if omitted the server will use the default value of "false"
    dataset = "" # str | Decides what type of trades to return, default is all trades and quotes, alternatively charting can be selected which will filter out unofficial trades. (optional)

    try:
        # Request returns data for the first trade (official and unofficial) after the specified time
        # example passing only required values which don't have defaults set
        # and optional values
        api_response = api_instance.tick_history_firsttrade_get(id, req_id=req_id, start_date=start_date, start_time=start_time, format=format, include_request=include_request, dataset=dataset)

        pprint(api_response)
    except fds.sdk.FactSetIntradayTickHistory.ApiException as e:
        print("Exception when calling FirstTradeApi->tick_history_firsttrade_get: %s\n" % e)

    # # Get response, http status code and response headers
    # try:
    #     # Request returns data for the first trade (official and unofficial) after the specified time
    #     api_response, http_status_code, response_headers = api_instance.tick_history_firsttrade_get_with_http_info(id, req_id=req_id, start_date=start_date, start_time=start_time, format=format, include_request=include_request, dataset=dataset)


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.FactSetIntradayTickHistory.ApiException as e:
    #     print("Exception when calling FirstTradeApi->tick_history_firsttrade_get: %s\n" % e)

    # # Get response asynchronous
    # try:
    #     # Request returns data for the first trade (official and unofficial) after the specified time
    #     async_result = api_instance.tick_history_firsttrade_get_async(id, req_id=req_id, start_date=start_date, start_time=start_time, format=format, include_request=include_request, dataset=dataset)
    #     api_response = async_result.get()


    #     pprint(api_response)
    # except fds.sdk.FactSetIntradayTickHistory.ApiException as e:
    #     print("Exception when calling FirstTradeApi->tick_history_firsttrade_get: %s\n" % e)

    # # Get response, http status code and response headers asynchronous
    # try:
    #     # Request returns data for the first trade (official and unofficial) after the specified time
    #     async_result = api_instance.tick_history_firsttrade_get_with_http_info_async(id, req_id=req_id, start_date=start_date, start_time=start_time, format=format, include_request=include_request, dataset=dataset)
    #     api_response, http_status_code, response_headers = async_result.get()


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.FactSetIntradayTickHistory.ApiException as e:
    #     print("Exception when calling FirstTradeApi->tick_history_firsttrade_get: %s\n" % e)

```

### Using Pandas

To convert an API response to a Pandas DataFrame, it is necessary to transform it first to a dictionary.
```python
import pandas as pd

response_dict = api_response.to_dict()['data']

simple_json_response = pd.DataFrame(response_dict)
nested_json_response = pd.json_normalize(response_dict)
```

### Debugging

The SDK uses the standard library [`logging`](https://docs.python.org/3/library/logging.html#module-logging) module.

Setting `debug` to `True` on an instance of the `Configuration` class sets the log-level of related packages to `DEBUG`
and enables additional logging in Pythons [HTTP Client](https://docs.python.org/3/library/http.client.html).

**Note**: This prints out sensitive information (e.g. the full request and response). Use with care.

```python
import logging
import fds.sdk.FactSetIntradayTickHistory

logging.basicConfig(level=logging.DEBUG)

configuration = fds.sdk.FactSetIntradayTickHistory.Configuration(...)
configuration.debug = True
```

### Configure a Proxy

You can pass proxy settings to the Configuration class:

* `proxy`: The URL of the proxy to use.
* `proxy_headers`: a dictionary to pass additional headers to the proxy (e.g. `Proxy-Authorization`).

```python
import fds.sdk.FactSetIntradayTickHistory

configuration = fds.sdk.FactSetIntradayTickHistory.Configuration(
    # ...
    proxy="http://secret:password@localhost:5050",
    proxy_headers={
        "Custom-Proxy-Header": "Custom-Proxy-Header-Value"
    }
)
```

### Custom SSL Certificate

TLS/SSL certificate verification can be configured with the following Configuration parameters:

* `ssl_ca_cert`: a path to the certificate to use for verification in `PEM` format.
* `verify_ssl`: setting this to `False` disables the verification of certificates.
  Disabling the verification is not recommended, but it might be useful during
  local development or testing.

```python
import fds.sdk.FactSetIntradayTickHistory

configuration = fds.sdk.FactSetIntradayTickHistory.Configuration(
    # ...
    ssl_ca_cert='/path/to/ca.pem'
)
```

### Request Retries

In case the request retry behaviour should be customized, it is possible to pass a `urllib3.Retry` object to the `retry` property of the Configuration.

```python
from urllib3 import Retry
import fds.sdk.FactSetIntradayTickHistory

configuration = fds.sdk.FactSetIntradayTickHistory.Configuration(
    # ...
)

configuration.retries = Retry(total=3, status_forcelist=[500, 502, 503, 504])
```


## Documentation for API Endpoints

All URIs are relative to *https://api-sandbox.factset.com*

Class | Method | HTTP request | Description
------------ | ------------- | ------------- | -------------
*FirstTradeApi* | [**tick_history_firsttrade_get**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetIntradayTickHistory/v1/docs/FirstTradeApi.md#tick_history_firsttrade_get) | **GET** /TickHistory/firsttrade | Request returns data for the first trade (official and unofficial) after the specified time
*LastTradeApi* | [**tick_history_lasttrade_get**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetIntradayTickHistory/v1/docs/LastTradeApi.md#tick_history_lasttrade_get) | **GET** /TickHistory/lasttrade | Request returns data for the last trade and associated quotes before the specified time
*QuoteAtTimeApi* | [**tick_history_qat_get**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetIntradayTickHistory/v1/docs/QuoteAtTimeApi.md#tick_history_qat_get) | **GET** /TickHistory/qat | Request returns a quote at the specified time
*TickHistoryApi* | [**get_tick_history**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetIntradayTickHistory/v1/docs/TickHistoryApi.md#get_tick_history) | **GET** /TickHistory/history | Tick History service returns trade data and associated quotes for a client specified interval and time period
*TradesAtTimeApi* | [**tick_history_tradesattime_get**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetIntradayTickHistory/v1/docs/TradesAtTimeApi.md#tick_history_tradesattime_get) | **GET** /TickHistory/tradesattime | Request returns the before and after trade data along with the associated quotes for the specified time


## Documentation For Models

 - [Fields](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetIntradayTickHistory/v1/docs/Fields.md)
 - [Intervals](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetIntradayTickHistory/v1/docs/Intervals.md)
 - [Specialfield](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetIntradayTickHistory/v1/docs/Specialfield.md)
 - [TickhistoryResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetIntradayTickHistory/v1/docs/TickhistoryResponse.md)


## Documentation For Authorization


## FactSetApiKey

- **Type**: HTTP basic authentication


## FactSetOAuth2

- **Type**: OAuth
- **Flow**: application
- **Authorization URL**: 
- **Scopes**: N/A


## Notes for Large OpenAPI documents
If the OpenAPI document is large, imports in fds.sdk.FactSetIntradayTickHistory.apis and fds.sdk.FactSetIntradayTickHistory.models may fail with a
RecursionError indicating the maximum recursion limit has been exceeded. In that case, there are a couple of solutions:

Solution 1:
Use specific imports for apis and models like:
- `from fds.sdk.FactSetIntradayTickHistory.api.default_api import DefaultApi`
- `from fds.sdk.FactSetIntradayTickHistory.model.pet import Pet`

Solution 2:
Before importing the package, adjust the maximum recursion limit as shown below:
```
import sys
sys.setrecursionlimit(1500)
import fds.sdk.FactSetIntradayTickHistory
from fds.sdk.FactSetIntradayTickHistory.apis import *
from fds.sdk.FactSetIntradayTickHistory.models import *
```

## Contributing

Please refer to the [contributing guide](../../../../CONTRIBUTING.md).

## Copyright

Copyright 2022 FactSet Research Systems Inc

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

